@@ -1,10 +1,11 @@
 import os
+import sys
 
 from argparse_prompt import PromptParser
 
-from migration_tools.library_configuration import LibraryConfiguration
-from migration_tools.migration_tasks import *  # pylint: disable=wildcard-import, unused-wildcard-import
-from migration_tools.migration_tasks import migration_task_base
+from folio_migration_tools.library_configuration import LibraryConfiguration
+from folio_migration_tools.migration_tasks import *  # noqa: 403
+from folio_migration_tools.migration_tasks import migration_task_base
 
 
 def parse_args():
@@ -16,17 +17,14 @@ def parse_args():
 
 def main():
     args = parse_args()
-    with open(
-        os.path.join(args.results_path, "LibraryConfigurationSchema.json"), "w"
-    ) as outfile:
+    with open(os.path.join(args.results_path, "LibraryConfigurationSchema.json"), "w") as outfile:
         outfile.write(LibraryConfiguration.schema_json(indent=4))
 
     for t in inheritors(migration_task_base.MigrationTaskBase):
-        with open(
-            os.path.join(args.results_path, f"{t.__name__}Schema.json"), "w"
-        ) as outfile:
+        with open(os.path.join(args.results_path, f"{t.__name__}Schema.json"), "w") as outfile:
             outfile.write(t.TaskConfiguration.schema_json(indent=4))
     print("done generating schemas.")
+    sys.exit(0)
 
 
 def inheritors(base_class):
