@@ -4,19 +4,17 @@ import sys
 import time
 import traceback
 from datetime import datetime
+from typing import List
+from typing import Optional
 from uuid import uuid4
 
 import requests
-from folio_uuid.folio_namespaces import FOLIONamespaces
-from folio_migration_tools.custom_exceptions import (
-    TransformationProcessError,
-    TransformationRecordFailedError,
-)
-from folio_migration_tools.library_configuration import (
-    FileDefinition,
-    LibraryConfiguration,
-)
+from folio_migration_tools.custom_exceptions import TransformationProcessError
+from folio_migration_tools.custom_exceptions import TransformationRecordFailedError
+from folio_migration_tools.library_configuration import FileDefinition
+from folio_migration_tools.library_configuration import LibraryConfiguration
 from folio_migration_tools.migration_tasks.migration_task_base import MigrationTaskBase
+from folio_uuid.folio_namespaces import FOLIONamespaces
 from pydantic import BaseModel
 
 
@@ -30,8 +28,9 @@ class BatchPoster(MigrationTaskBase):
         name: str
         migration_task_type: str
         object_type: str
-        file: FileDefinition
+        files: List[FileDefinition]
         batch_size: int
+        rerun_failed_records: Optional[bool] = True
 
     @staticmethod
     def get_object_type() -> FOLIONamespaces:
@@ -41,7 +40,7 @@ class BatchPoster(MigrationTaskBase):
         self,
         task_config: TaskConfiguration,
         library_config: LibraryConfiguration,
-        use_logging: bool = True
+        use_logging: bool = True,
     ):
         super().__init__(library_config, task_config, use_logging)
         self.task_config = task_config
@@ -50,12 +49,6 @@ class BatchPoster(MigrationTaskBase):
         self.api_path = list_objects(self.task_config.object_type)
         self.snapshot_id = str(uuid4())
         self.failed_objects = []
-        object_name_formatted = self.task_config.object_type.replace(" ", "").lower()
-        time_stamp = time.strftime("%Y%m%d-%H%M%S")
-        self.failed_recs_path = (
-            self.folder_structure.results_folder
-            / f"failed_{object_name_formatted}_records_{time_stamp}.json"
-        )
         self.batch_size = self.task_config.batch_size
         logging.info("Batch size is %s", self.batch_size)
         self.processed = 0
@@ -71,49 +64,50 @@ class BatchPoster(MigrationTaskBase):
     def do_work(self):
         try:
             batch = []
-            path = (
-                self.folder_structure.results_folder / self.task_config.file.file_name
-            )
             if self.task_config.object_type == "SRS":
                 self.create_snapshot()
-            with open(path) as rows, open(
-                self.failed_recs_path, "w"
-            ) as failed_recs_file:
-                last_row = ""
-                for num_records, row in enumerate(rows, start=1):
-                    last_row = row
-                    if row.strip():
-                        try:
-                            if self.task_config.object_type == "Extradata":
-                                self.post_extra_data(row, num_records, failed_recs_file)
-                            else:
-                                json_rec = json.loads(row.split("\t")[-1])
-                                if self.task_config.object_type == "SRS":
-                                    json_rec["snapshotId"] = self.snapshot_id
-                                if num_records == 1:
-                                    logging.info(json.dumps(json_rec, indent=True))
-                                batch.append(json_rec)
-                                if len(batch) == int(self.batch_size):
-                                    self.post_batch(
-                                        batch, failed_recs_file, num_records
+            with open(self.folder_structure.failed_recs_path, "w") as failed_recs_file:
+                for file_def in self.task_config.files:
+                    path = self.folder_structure.results_folder / file_def.file_name
+                    with open(path) as rows:
+                        logging.info("Running %s", path)
+                        last_row = ""
+                        for num_records, row in enumerate(rows, start=1):
+                            last_row = row
+                            if row.strip():
+                                try:
+                                    if self.task_config.object_type == "Extradata":
+                                        self.post_extra_data(row, num_records, failed_recs_file)
+                                    else:
+                                        json_rec = json.loads(row.split("\t")[-1])
+                                        if self.task_config.object_type == "SRS":
+                                            json_rec["snapshotId"] = self.snapshot_id
+                                        if num_records == 1:
+                                            logging.info(json.dumps(json_rec, indent=True))
+                                        batch.append(json_rec)
+                                        if len(batch) == int(self.batch_size):
+                                            self.post_batch(batch, failed_recs_file, num_records)
+                                            batch = []
+                                except UnicodeDecodeError as unicode_error:
+                                    self.handle_unicode_error(unicode_error, last_row)
+                                except TransformationProcessError as tpe:
+                                    self.handle_generic_exception(
+                                        tpe,
+                                        last_row,
+                                        batch,
+                                        num_records,
+                                        failed_recs_file,
+                                    )
+                                    logging.critical("Halting %s", tpe)
+                                except TransformationRecordFailedError as exception:
+                                    self.handle_generic_exception(
+                                        exception,
+                                        last_row,
+                                        batch,
+                                        num_records,
+                                        failed_recs_file,
                                     )
                                     batch = []
-                        except UnicodeDecodeError as unicode_error:
-                            self.handle_unicode_error(unicode_error, last_row)
-                        except TransformationProcessError as tpe:
-                            self.handle_generic_exception(
-                                tpe, last_row, batch, num_records, failed_recs_file
-                            )
-                            logging.critical("Halting %s", tpe)
-                        except TransformationRecordFailedError as exception:
-                            self.handle_generic_exception(
-                                exception,
-                                last_row,
-                                batch,
-                                num_records,
-                                failed_recs_file,
-                            )
-                            batch = []
 
                 if self.task_config.object_type != "Extradata" and any(batch):
                     try:
@@ -139,19 +133,12 @@ class BatchPoster(MigrationTaskBase):
         elif response.status_code == 422:
             self.num_failures += 1
             error_msg = json.loads(response.text)["errors"][0]["message"]
-            logging.error(
-                "Row %s\tHTTP %s\t %s", num_records, response.status_code, error_msg
-            )
-            if (
-                "id value already exists"
-                not in json.loads(response.text)["errors"][0]["message"]
-            ):
+            logging.error("Row %s\tHTTP %s\t %s", num_records, response.status_code, error_msg)
+            if "id value already exists" not in json.loads(response.text)["errors"][0]["message"]:
                 failed_recs_file.write(row)
         else:
             self.num_failures += 1
-            logging.error(
-                "Row %s\tHTTP %s\t%s", num_records, response.status_code, response.text
-            )
+            logging.error("Row %s\tHTTP %s\t%s", num_records, response.status_code, response.text)
             failed_recs_file.write(row)
         if num_records % 50 == 0:
             logging.info(
@@ -165,22 +152,18 @@ class BatchPoster(MigrationTaskBase):
             url, headers=self.folio_client.okapi_headers, data=body.encode("utf-8")
         )
 
-    def handle_generic_exception(
-        self, exception, last_row, batch, num_records, failed_recs_file
-    ):
+    def handle_generic_exception(self, exception, last_row, batch, num_records, failed_recs_file):
         logging.error("%s", exception)
         # logging.error("Failed row: %s", last_row)
         self.failed_batches += 1
         self.failed_records += len(batch)
         write_failed_batch_to_file(batch, failed_recs_file)
-        logging.info(
-            "Resetting batch...Number of failed batches: %s", self.failed_batches
-        )
+        logging.info("Resetting batch...Number of failed batches: %s", self.failed_batches)
         batch = []
         if self.failed_batches > 50000:
             logging.error("Exceeded number of failed batches at row %s", num_records)
             logging.critical("Halting")
-            sys.exit()
+            sys.exit(1)
 
     def handle_unicode_error(self, unicode_error, last_row):
         logging.info("=========ERROR==============")
@@ -258,13 +241,11 @@ class BatchPoster(MigrationTaskBase):
             )
         elif response.status_code == 400:
             # Likely a json parsing error
-            print(response.text)
-            raise TransformationProcessError(
-                "", "HTTP 400. Somehting is wrong. Quitting"
-            )
+            logging.error(response.text)
+            raise TransformationProcessError("", "HTTP 400. Somehting is wrong. Quitting")
         elif self.task_config.object_type == "SRS" and response.status_code == 500:
             logging.info(
-                "Post failed. Size: %s Waiting 30 seconds until reposting. Number of tries: %s of 5 before failing batch",
+                "Post failed. Size: %s Waiting 30s until reposting. Number of tries: %s of 5",
                 get_req_size(response),
                 recursion_depth,
             )
@@ -279,15 +260,13 @@ class BatchPoster(MigrationTaskBase):
                     response.text,
                 )
             else:
-                self.post_batch(
-                    batch, failed_recs_file, num_records, recursion_depth + 1
-                )
+                self.post_batch(batch, failed_recs_file, num_records, recursion_depth + 1)
         else:
             try:
-                print(response.text)
+                logging.info(response.text)
                 resp = json.dumps(response, indent=4)
-            except:
-                print()
+            except Exception:
+                logging.exception("something unexpected happened")
                 resp = response
             raise TransformationRecordFailedError(
                 "",
@@ -313,6 +292,8 @@ class BatchPoster(MigrationTaskBase):
 
     def wrap_up(self):
         logging.info("Done. Wrapping up")
+        if self.task_config.object_type == "SRS":
+            self.commit_snapshot()
         if self.task_config.object_type != "Extradata":
             logging.info(
                 (
@@ -321,22 +302,38 @@ class BatchPoster(MigrationTaskBase):
                 ),
                 self.failed_records,
                 self.failed_batches,
-                self.failed_recs_path,
+                self.folder_structure.failed_recs_path,
             )
+
         else:
-            logging.info(
-                "Done posting % records. % failed", self.num_posted, self.num_failures
-            )
-        if self.task_config.object_type == "SRS":
-            self.commit_snapshot()
+            logging.info("Done posting % records. % failed", self.num_posted, self.num_failures)
+
+        self.rerun_run()
+
+    def rerun_run(self):
+        if self.task_config.rerun_failed_records and (
+            self.failed_records > 0 or self.num_failures > 0
+        ):
+            logging.info("Rerunning the failed records from the load with a batchsize of 1")
+            try:
+                self.task_config.batch_size = 1
+                self.task_config.files = [
+                    FileDefinition(file_name=str(self.folder_structure.failed_recs_path.name))
+                ]
+                self.task_config.rerun_failed_records = False
+                self.__init__(self.task_config, self.library_configuration)
+                self.do_work()
+                self.wrap_up()
+                logging.info("Done rerunning the posting")
+            except Exception as ee:
+                logging.exception("Happed during rerun")
+                raise TransformationProcessError("Error during rerun") from ee
 
     def create_snapshot(self):
         snapshot = {
             "jobExecutionId": self.snapshot_id,
             "status": "PARSING_IN_PROGRESS",
-            "processingStartedDate": datetime.utcnow().isoformat(
-                timespec="milliseconds"
-            ),
+            "processingStartedDate": datetime.utcnow().isoformat(timespec="milliseconds"),
         }
         try:
             url = f"{self.folio_client.okapi_url}/source-storage/snapshots"
@@ -357,7 +354,7 @@ class BatchPoster(MigrationTaskBase):
                     logging.info(res.status_code)
         except Exception:
             logging.exception("Could not post the snapshot")
-            sys.exit()
+            sys.exit(1)
 
     def commit_snapshot(self):
         snapshot = {"jobExecutionId": self.snapshot_id, "status": "COMMITTED"}
@@ -367,17 +364,15 @@ class BatchPoster(MigrationTaskBase):
                 url, data=json.dumps(snapshot), headers=self.folio_client.okapi_headers
             )
             res.raise_for_status()
-            logging.info(
-                "Posted Committed snapshot to FOLIO: %s", json.dumps(snapshot, indent=4)
-            )
+            logging.info("Posted Committed snapshot to FOLIO: %s", json.dumps(snapshot, indent=4))
         except Exception:
             logging.exception(
-                "Could not commit snapshot with id %s. Post the following to /source-storage/snapshots/%s:",
+                "Could not commit snapshot with id %s. Post this to /source-storage/snapshots/%s:",
                 self.snapshot_id,
                 self.snapshot_id,
             )
             logging.info("%s", json.dumps(snapshot, indent=4))
-            sys.exit()
+            sys.exit(1)
 
 
 def list_objects(object_type: str):
@@ -424,9 +419,9 @@ def list_objects(object_type: str):
         return choices[object_type]
     except KeyError:
         key_string = ",".join(choices.keys())
-        print("", f"Wrong type. Only one of {key_string} are allowed")
-        print("Halting")
-        sys.exit()
+        logging.error(f"Wrong type. Only one of {key_string} are allowed")
+        logging.error("Halting")
+        sys.exit(1)
 
 
 def chunks(records, number_of_chunks):
