@@ -10,6 +10,7 @@ import traceback
 import uuid
 from os.path import isfile
 from typing import List, Optional
+from folioclient import FolioClient
 
 from folio_uuid import FolioUUID
 from folio_uuid.folio_namespaces import FOLIONamespaces
@@ -79,14 +80,12 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 self.load_mapped_fields(),
                 self.load_location_map(),
                 self.load_call_number_type_map(),
-                self.load_id_map(self.folder_structure.instance_id_map_path),
+                self.load_id_map(self.folder_structure.instance_id_map_path, True),
                 library_config,
             )
             self.holdings = {}
             self.total_records = 0
-            self.holdings_id_map = self.load_id_map(
-                self.folder_structure.holdings_id_map_path
-            )
+            self.holdings_id_map = self.load_id_map(self.folder_structure.holdings_id_map_path)
             self.holdings_sources = self.get_holdings_sources()
             self.results_path = self.folder_structure.created_objects_path
             self.holdings_types = list(
@@ -124,12 +123,13 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                             self.task_config.holdings_type_uuid_for_boundwiths,
                         )
                     )
+
             else:
                 logging.info("No file of legacy holdings setup.")
         except TransformationProcessError as process_error:
             logging.critical(process_error)
             logging.critical("Halting.")
-            sys.exit()
+            sys.exit(1)
         except Exception as exception:
             logging.info("\n=======ERROR===========")
             logging.info(exception)
@@ -149,8 +149,7 @@ class HoldingsCsvTransformer(MigrationTaskBase):
 
     def load_location_map(self):
         with open(
-            self.folder_structure.mapping_files_folder
-            / self.task_config.location_map_file_name
+            self.folder_structure.mapping_files_folder / self.task_config.location_map_file_name
         ) as location_map_f:
             return self.load_ref_data_map_from_file(
                 location_map_f, "Found %s rows in location map"
@@ -164,13 +163,10 @@ class HoldingsCsvTransformer(MigrationTaskBase):
 
     def load_mapped_fields(self):
         with open(
-            self.folder_structure.mapping_files_folder
-            / self.task_config.holdings_map_file_name
+            self.folder_structure.mapping_files_folder / self.task_config.holdings_map_file_name
         ) as holdings_mapper_f:
             holdings_map = json.load(holdings_mapper_f)
-            logging.info(
-                "%s fields in holdings mapping file map", len(holdings_map["data"])
-            )
+            logging.info("%s fields in holdings mapping file map", len(holdings_map["data"]))
             mapped_fields = MappingFileMapperBase.get_mapped_folio_properties_from_map(
                 holdings_map
             )
@@ -200,9 +196,7 @@ class HoldingsCsvTransformer(MigrationTaskBase):
 
     def load_instance_id_map(self):
         res = {}
-        with open(
-            self.folder_structure.instance_id_map_path, "r"
-        ) as instance_id_map_file:
+        with open(self.folder_structure.instance_id_map_path, "r") as instance_id_map_file:
             for index, json_string in enumerate(instance_id_map_file):
                 # Format:{"legacy_id", "folio_id","instanceLevelCallNumber"}
                 if index % 100000 == 0:
@@ -225,21 +219,17 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 )
                 logging.critical(error_str)
                 print(f"\n{error_str}\nHalting")
-                sys.exit()
-        logging.info(  # pylint: disable=logging-fstring-interpolation
-            f"processed {self.total_records:,} records in {len(self.files)} files"
-        )
+                sys.exit(1)
+        logging.info(f"processed {self.total_records:,} records in {len(self.files)} files")
 
     def wrap_up(self):
-        logging.info("Done. Wrapping up...")
+        logging.info("Work done. Wrapping up...")
         if any(self.holdings):
             logging.info(
                 "Saving holdings created to %s",
                 self.folder_structure.created_objects_path,
             )
-            with open(
-                self.folder_structure.created_objects_path, "w+"
-            ) as holdings_file:
+            with open(self.folder_structure.created_objects_path, "w+") as holdings_file:
                 for holding in self.holdings.values():
                     for legacy_id in holding["formerIds"]:
                         # Prevent the first item in a boundwith to be overwritten
@@ -255,9 +245,7 @@ class HoldingsCsvTransformer(MigrationTaskBase):
             self.mapper.save_id_map_file(
                 self.folder_structure.holdings_id_map_path, self.holdings_id_map
             )
-        with open(
-            self.folder_structure.migration_reports_file, "w"
-        ) as migration_report_file:
+        with open(self.folder_structure.migration_reports_file, "w") as migration_report_file:
             logging.info(
                 "Writing migration- and mapping report to %s",
                 self.folder_structure.migration_reports_file,
@@ -274,13 +262,9 @@ class HoldingsCsvTransformer(MigrationTaskBase):
     def validate_merge_criterias(self):
         holdings_schema = self.folio_client.get_holdings_schema()
         properties = holdings_schema["properties"].keys()
-        print(properties)
-        print(self.task_config.holdings_merge_criteria)
-        res = [
-            mc
-            for mc in self.task_config.holdings_merge_criteria
-            if mc not in properties
-        ]
+        logging.info(properties)
+        logging.info(self.task_config.holdings_merge_criteria)
+        res = [mc for mc in self.task_config.holdings_merge_criteria if mc not in properties]
         if any(res):
             logging.critical(
                 (
@@ -289,20 +273,17 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 ),
                 ", ".join(res),
             )
-            sys.exit()
+            sys.exit(1)
 
     def process_single_file(self, file_name):
         with open(file_name, encoding="utf-8-sig") as records_file:
-            self.mapper.migration_report.add_general_statistics(
-                "Number of files processed"
-            )
+            self.mapper.migration_report.add_general_statistics("Number of files processed")
             start = time.time()
             records_processed = 0
-            for idx, legacy_record in enumerate(
-                self.mapper.get_objects(records_file, file_name)
-            ):
+            for idx, legacy_record in enumerate(self.mapper.get_objects(records_file, file_name)):
                 records_processed = idx + 1
                 try:
+                    self.mapper.verify_legacy_record(legacy_record, idx)
                     folio_rec, legacy_id = self.mapper.do_map(
                         legacy_record, f"row # {idx}", FOLIONamespaces.holdings
                     )
@@ -319,11 +300,9 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 if idx > 1 and idx % 10000 == 0:
                     elapsed = idx / (time.time() - start)
                     elapsed_formatted = "{0:.4g}".format(elapsed)
-                    logging.info(  # pylint: disable=logging-fstring-interpolation
-                        f"{idx:,} records processed. Recs/sec: {elapsed_formatted} "
-                    )
+                    logging.info(f"{idx:,} records processed. Recs/sec: {elapsed_formatted} ")
             self.total_records = records_processed
-            logging.info(  # pylint: disable=logging-fstring-interpolation
+            logging.info(
                 f"Done processing {file_name} containing {self.total_records:,} records. "
                 f"Total records processed: {self.total_records:,}"
             )
@@ -337,18 +316,15 @@ class HoldingsCsvTransformer(MigrationTaskBase):
 
         holdings_from_row = []
         all_instance_ids = folio_rec.get("instanceId", [])
-        if len(all_instance_ids) == 1:  # Normal case.
+        if len(all_instance_ids) == 1:
+            # Normal case.
             folio_rec["instanceId"] = folio_rec["instanceId"][0]
             holdings_from_row.append(folio_rec)
 
         elif len(folio_rec.get("instanceId", [])) > 1:  # Bound-with.
-            holdings_from_row.extend(
-                self.create_bound_with_holdings(folio_rec, legacy_id)
-            )
+            holdings_from_row.extend(self.create_bound_with_holdings(folio_rec, legacy_id))
         else:
-            raise TransformationRecordFailedError(
-                legacy_id, "No instance id in parsed record", ""
-            )
+            raise TransformationRecordFailedError(legacy_id, "No instance id in parsed record", "")
 
         for folio_holding in holdings_from_row:
             self.merge_holding_in(folio_holding, all_instance_ids, legacy_id)
@@ -365,17 +341,9 @@ class HoldingsCsvTransformer(MigrationTaskBase):
         # Add former ids
         temp_ids = []
         for former_id in folio_holding.get("formerIds", []):
-            if (
-                former_id.startswith("[")
-                and former_id.endswith("]")
-                and "," in former_id
-            ):
+            if former_id.startswith("[") and former_id.endswith("]") and "," in former_id:
                 ids = list(
-                    former_id[1:-1]
-                    .replace('"', "")
-                    .replace(" ", "")
-                    .replace("'", "")
-                    .split(",")
+                    former_id[1:-1].replace('"', "").replace(" ", "").replace("'", "").split(",")
                 )
                 temp_ids.extend(ids)
             else:
@@ -412,52 +380,64 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                     f'{folio_holding["id"]}-{instance_id}',
                 )
             )
-            self.mapper.migration_report.add_general_statistics(
-                "Bound-with holdings created"
-            )
+            self.mapper.migration_report.add_general_statistics("Bound-with holdings created")
             yield bound_with_holding
 
-    def generate_boundwith_part(self, legacy_id: str, bound_with_holding: dict):
+    @staticmethod
+    def generate_boundwith_part(
+        folio_client: FolioClient, legacy_item_id: str, bound_with_holding: dict
+    ):
         part = {
             "id": str(uuid.uuid4()),
             "holdingsRecordId": bound_with_holding["id"],
             "itemId": str(
                 FolioUUID(
-                    self.folio_client.okapi_url,
+                    folio_client.okapi_url,
                     FOLIONamespaces.items,
-                    legacy_id,
+                    legacy_item_id,
                 )
             ),
         }
         logging.log(25, f"boundwithPart\t{json.dumps(part)}")
 
     def merge_holding_in(
-        self, new_folio_holding: dict, instance_ids: list, legacy_id: str
+        self, incoming_holding: dict, instance_ids: list[str], legacy_item_id: str
     ):
-        if len(instance_ids) > 1:  # Is boundwith
-            bw_key = (
-                f"bw_{new_folio_holding['instanceId']}_{'_'.join(sorted(instance_ids))}"
-            )
+        """Determines what newly generated holdingsrecords are to be merged with
+        previously created ones. When that is done, it generates the correct boundwith
+        parts needed.
+
+        Args:
+            new_folio_holding (dict): The newly created FOLIO Holding
+            instance_ids (list): the instance IDs tied to the current item
+            legacy_item_id (str): Id of the Item the holding was generated from
+        """
+
+        if len(instance_ids) > 1:
+            # Is boundwith
+            bw_key = f"bw_{incoming_holding['instanceId']}_{'_'.join(sorted(instance_ids))}"
             if bw_key not in self.bound_with_keys:
                 self.bound_with_keys.add(bw_key)
-                self.holdings[bw_key] = new_folio_holding
-                self.generate_boundwith_part(legacy_id, new_folio_holding)
+                self.holdings[bw_key] = incoming_holding
+                self.generate_boundwith_part(self.folio_client, legacy_item_id, incoming_holding)
                 self.mapper.migration_report.add_general_statistics(
                     "Unique BW Holdings created from Items"
                 )
-
             else:
+                self.merge_holding(bw_key, incoming_holding)
+                self.generate_boundwith_part(
+                    self.folio_client, legacy_item_id, self.holdings[bw_key]
+                )
+                self.holdings_id_map[legacy_item_id] = self.mapper.get_id_map_dict(
+                    legacy_item_id, self.holdings[bw_key]
+                )
                 self.mapper.migration_report.add_general_statistics(
                     "BW Items found tied to previously created BW Holdings"
                 )
-                self.merge_holding(bw_key, new_folio_holding)
-                self.generate_boundwith_part(legacy_id, self.holdings[bw_key])
-                self.holdings_id_map[legacy_id] = self.mapper.get_id_map_dict(
-                    legacy_id, self.holdings[bw_key]
-                )
-        else:  # Regular
+        else:
+            # Regular holding. Merge according to criteria
             new_holding_key = HoldingsHelper.to_key(
-                new_folio_holding,
+                incoming_holding,
                 self.task_config.holdings_merge_criteria,
                 self.mapper.migration_report,
                 self.task_config.holdings_type_uuid_for_boundwiths,
@@ -466,12 +446,12 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                 self.mapper.migration_report.add_general_statistics(
                     "Holdings already created from Item"
                 )
-                self.merge_holding(new_holding_key, new_folio_holding)
+                self.merge_holding(new_holding_key, incoming_holding)
             else:
                 self.mapper.migration_report.add_general_statistics(
                     "Unique Holdings created from Items"
                 )
-                self.holdings[new_holding_key] = new_folio_holding
+                self.holdings[new_holding_key] = incoming_holding
 
     def merge_holding(self, holdings_key: str, new_holdings_record: dict):
         self.holdings[holdings_key] = HoldingsHelper.merge_holding(
@@ -486,18 +466,14 @@ class HoldingsCsvTransformer(MigrationTaskBase):
                     "/holdings-sources", "holdingsRecordsSources"
                 )
             )
-            logging.info(
-                "Fetched %s holdingsRecordsSources from tenant", len(holdings_sources)
-            )
+            logging.info("Fetched %s holdingsRecordsSources from tenant", len(holdings_sources))
             res = {n["name"].upper(): n["id"] for n in holdings_sources}
             if "FOLIO" not in res:
                 raise TransformationProcessError(
                     "", "No holdings source with name FOLIO in tenant"
                 )
             if "MARC" not in res:
-                raise TransformationProcessError(
-                    "", "No holdings source with name MARC in tenant"
-                )
+                raise TransformationProcessError("", "No holdings source with name MARC in tenant")
         return res
 
 
